// MythRemote - copyright David Kuder (GrizzlyAdams) 2007
// Based on DSWifi Test Project - copyright Stephen Stair 2005
// Based on template project in ndslib by wntrmute
/****************************************************************************** 
DSWifi Lib and test materials are licenced under the MIT open source licence:
Copyright (c) 2005 Stephen Stair

MythRemote is also licensed under the MIT open source license:
Copyright (c) 2007 David Kuder

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
******************************************************************************/


#include <nds.h>

#include <nds/bios.h>
#include <nds/arm7/touch.h>
#include <nds/arm7/clock.h>

#include <cartreset.h>

#include <dswifi7.h>

#include "mythremoteipc.h"

#define PM_SOUND			0x01
#define PM_BOTTOMLIGHT		0x04
#define PM_TOPLIGHT		0x08
#define PM_LED_SLOW		0x10
#define PM_LED_FAST		0x30
#define PM_OFF			0x40

typedef void(*call0)(void);
typedef void(*call3)(u32,u32,u32);

//---------------------------------------------------------------------------------
void startSound(int sampleRate, const void* data, u32 bytes, u8 channel, u8 vol,  u8 pan, u8 format) {
//---------------------------------------------------------------------------------
	SCHANNEL_TIMER(channel)  = SOUND_FREQ(sampleRate);
	SCHANNEL_SOURCE(channel) = (u32)data;
	SCHANNEL_LENGTH(channel) = bytes >> 2 ;
	SCHANNEL_CR(channel)     = SCHANNEL_ENABLE | SOUND_ONE_SHOT | SOUND_VOL(vol) | SOUND_PAN(pan) | (format==1?SOUND_8BIT:SOUND_16BIT);
}


//---------------------------------------------------------------------------------
s32 getFreeSoundChannel() {
//---------------------------------------------------------------------------------
	int i;
	for (i=0; i<16; i++) {
		if ( (SCHANNEL_CR(i) & SCHANNEL_ENABLE) == 0 ) return i;
	}
	return -1;
}


//---------------------------------------------------------------------------------
void VblankHandler(void) {
//---------------------------------------------------------------------------------
	static int heartbeat = 0;

	uint16 but=0, x=0, y=0, xpx=0, ypx=0, z1=0, z2=0, batt=0, aux=0;
	int t1=0, t2=0;
	uint32 temp=0;
	uint8 ct[sizeof(IPC->curtime)];
	u32 i;

	// Update the heartbeat
	heartbeat++;

	// Read the touch screen

	but = REG_KEYXY;

	if (!(but & (1<<6))) {

		touchPosition tempPos = touchReadXY();

		x = tempPos.x;
		y = tempPos.y;
		xpx = tempPos.px;
		ypx = tempPos.py;
	}

	z1 = touchRead(TSC_MEASURE_Z1);
	z2 = touchRead(TSC_MEASURE_Z2);

	
	batt = touchRead(TSC_MEASURE_BATTERY);
	aux  = touchRead(TSC_MEASURE_AUX);

	// Read the time
	rtcGetTime((uint8 *)ct);
	BCDToInteger((uint8 *)&(ct[1]), 7);

	// Read the temperature
	temp = touchReadTemperature(&t1, &t2);

	// Update the IPC struct
	IPC->heartbeat	= heartbeat;
	IPC->buttons		= but;
	IPC->touchX			= x;
	IPC->touchY			= y;
	IPC->touchXpx		= xpx;
	IPC->touchYpx		= ypx;
	IPC->touchZ1		= z1;
	IPC->touchZ2		= z2;
	IPC->battery		= batt;
	IPC->aux			= aux;

	for(i=0; i<sizeof(ct); i++) {
		IPC->curtime[i] = ct[i];
	}

	IPC->temperature = temp;
	IPC->tdiode1 = t1;
	IPC->tdiode2 = t2;


	//sound code  :)
	TransferSound *snd = IPC->soundData;
	IPC->soundData = 0;

	if (0 != snd) {

		for (i=0; i<snd->count; i++) {
			s32 chan = getFreeSoundChannel();

			if (chan >= 0) {
				startSound(snd->data[i].rate, snd->data[i].data, snd->data[i].len, chan, snd->data[i].vol, snd->data[i].pan, snd->data[i].format);
			}
		}
	}

	Wifi_Update();
}

void writePM(int flags) {
	call3 spi_write=(call3)0x2369;
	spi_write(SPI_DEVICE_POWER,0,1);
	spi_write(SPI_DEVICE_POWER,flags,0);
}

void IPC_PowerOff() {
	writePM(PM_OFF);
}

int lights = PM_TOPLIGHT | PM_BOTTOMLIGHT;

void IPC_LidOpen() {
	writePM(lights | PM_SOUND);
}

void IPC_LidClose() {
	writePM(PM_SOUND);
}

void IPC_LCDOn(int LCD) {
	lights &= ~LCD;
	writePM(lights | PM_SOUND);
}

void IPC_LCDOff(int LCD) {
	lights |= LCD;
	writePM(lights | PM_SOUND);
}

void IPC_CartReset() {
	while (1) {
		if(*((vu32*)0x027FFE24) == (u32)0x027FFE04)
			cartExecute();
		swiWaitForVBlank();
	}
	
}

void IPC_Sleep() {
	u32 ie_save;
	writePM(PM_SOUND | PM_LED_SLOW);
	swiChangeSoundBias(0,0x400);
	ie_save=REG_IE;
	REG_IE=IRQ_LID;
	swiSleep();
	REG_IF=~0;
	REG_IE=ie_save;
	writePM(lights | PM_SOUND);
	swiChangeSoundBias(1,0x400);
}

//---------------------------------------------------------------------------------
// callback to allow wifi library to notify arm9
//---------------------------------------------------------------------------------
void arm7_synctoarm9() { // send fifo message
	while(REG_IPC_FIFO_CR & IPC_FIFO_SEND_FULL);
	REG_IPC_FIFO_TX = IPC_WIFISYNC9;
}
 
enum {
	CMD_WAIT,
	WIFI_INIT
};
 
u32 fifo_status = CMD_WAIT;
 
//---------------------------------------------------------------------------------
// interrupt handler to allow incoming notifications from arm9
//---------------------------------------------------------------------------------
void arm7_fifo() { // check incoming fifo messages
//---------------------------------------------------------------------------------
	while ( !(REG_IPC_FIFO_CR & (IPC_FIFO_RECV_EMPTY)) ) {
		u32 msg = REG_IPC_FIFO_RX;
		REG_IPC_FIFO_TX = msg;
 
		switch (fifo_status) {
			case WIFI_INIT:
				REG_IME = 1;	// allow other interrupts
				Wifi_Init(msg);
				Wifi_SetSyncHandler(arm7_synctoarm9); // allow wifi lib to notify arm9
				fifo_status = CMD_WAIT;
				break;

			case CMD_WAIT:
				switch(msg) {
					case IPC_WIFISYNC7:
						Wifi_Sync();
						break;

					case IPC_POWEROFF:
						IPC_PowerOff();
						break;

					case IPC_CARTRESET:
						IPC_CartReset();
						break;

					case IPC_SLEEP:
						IPC_Sleep();
						break;

					case IPC_LIDOPEN:
						IPC_LidOpen();
						break;

					case IPC_LIDCLOSE:
						IPC_LidClose();
						break;

					case IPC_TOPOFF:
						IPC_LCDOff(PM_TOPLIGHT);
						break;

					case IPC_TOPON:
						IPC_LCDOn(PM_TOPLIGHT);
						break;

					case IPC_BOTTOMOFF:
						IPC_LCDOff(PM_BOTTOMLIGHT);
						break;

					case IPC_BOTTOMON:
						IPC_LCDOn(PM_BOTTOMLIGHT);
						break;

					case IPC_WIFIINIT:
						fifo_status = WIFI_INIT;
						break;
				}
				break;
		}
	}
}




//---------------------------------------------------------------------------------
int main(int argc, char ** argv) {
//---------------------------------------------------------------------------------
	// Reset the clock if needed
	rtcReset();

	//enable sound
	powerON(POWER_SOUND);
	SOUND_CR = SOUND_ENABLE | SOUND_VOL(0x7F);
	IPC->soundData = 0;

	irqInit();
	irqSet(IRQ_VBLANK, VblankHandler);
	irqEnable(IRQ_VBLANK);

	irqSet(IRQ_WIFI, Wifi_Interrupt);
	irqEnable(IRQ_WIFI);

	REG_IPC_FIFO_CR = IPC_FIFO_ENABLE | IPC_FIFO_SEND_CLEAR | IPC_FIFO_RECV_IRQ;

	// Keep the ARM7 out of main RAM
	while (1) {
		arm7_fifo();
		swiWaitForVBlank();
	}
}


