// MythRemote - copyright David Kuder (GrizzlyAdams) 2007
// Based on DSWifi Test Project - copyright Stephen Stair 2005
// Based on template project in ndslib by wntrmute
/****************************************************************************** 
DSWifi Lib and test materials are licenced under the MIT open source licence:
Copyright (c) 2005 Stephen Stair

MythRemote is also licensed under the MIT open source license:
Copyright (c) 2007 David Kuder

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
******************************************************************************/


#include "nds.h"
#include <nds/arm9/console.h>		//basic print funcionality
#include <nds/arm9/image.h>		//needed to load pcx files


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>

#include <fat.h>
#include <cartreset.h>

#include "mythremote.h"

// Redefined this in the wifi libs so we need to deal with that.
#undef FD_SETSIZE

#include <dswifi9.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

Host_t Host[3] = {
	{ 0xc0a80801, 6546, HT_MYTH },
	{ 0xc0a80002, 6546, HT_MYTH },
	{ 0xc0a80003, 6546, HT_MYTH },
};

unsigned short UpdatePort=12345;

void * sgIP_malloc(int size) { return malloc(size); }
void sgIP_free(void * ptr) { free(ptr); }


touchPosition touchXY;

int DEBUG=0;
int NO3D=0;

int splash_fade=360;
Skin_t Skin;
unsigned short *TopScreen = VRAM2;
unsigned short *BottomScreen = VRAM1;
unsigned short BackBuffer[256*256];
int screen_on=1;

int framectr=0;


Button_t MainMenu_Buttons[] = {
	//"**------------------------**"
	{ 28,12,	200,16,
		libskin_BC_Normal, libskin_BS_Normal, 0,
		"MythFrontend 1", Do_Remote, (void *)&Host[0], Effect_Down },
	{ 28,36,	200,16, 
		libskin_BC_Normal, libskin_BS_Normal, 0,
		"MythFrontend 2", Do_Remote, (void *)&Host[1], Effect_Down },
	{ 28,60,	200,16, 
		libskin_BC_Normal, libskin_BS_Normal, 0,
		"MythFrontend 3", Do_Remote, (void *)&Host[2], Effect_Down },

	{ 28,108,	200,16, 
		libskin_BC_Normal, libskin_BS_Normal, 0,
		"Configure", Do_Config, &ConfigMenu, Effect_Down },

#if 0
	{ 180,132,72,16, 
		libskin_BC_Normal, libskin_BS_Normal, 0,
		"Sleep", Do_Sleep2, NULL, 0 },

	{ 12,156,	72,16, 
		libskin_BC_Normal, libskin_BS_Normal, 0,
		"Update", Do_Update, NULL, Effect_Fade },
#endif

	{ 12,156,72,16, 
		libskin_BC_Normal, libskin_BS_Normal, 0,
		"Power", Do_PowerOff, NULL, Effect_Fade },
	{ 172,156,72,16, 
		libskin_BC_Normal, libskin_BS_Normal, 0,
		"Reboot", Do_Reset, NULL, Effect_Fade },

	{ 0,0,	0,0, libskin_BC_Invalid, 0, 0,  NULL, NULL, NULL, 0 }
};


ButtonPanel_t MainMenu = {
	1,	// Accept Cursor Input
	0,	// Select 1st entry
	-1,	// TouchState
	0,	// Frames
	0,	// Bool Exit
	Effect_Fade,
	MainMenu_Buttons
};


//---------------------------------------------------------------------------------
int main(void) {
//---------------------------------------------------------------------------------
	powerON(POWER_ALL_2D); 

	irqInit();

	irqEnable(IRQ_VBLANK);

	// set the mode for 2 text layers and two extended background layers
	videoSetMode(MODE_5_2D | DISPLAY_BG3_ACTIVE);


	// set the first bank as background memory and the third as sub background memory
	// B and D are not used (if you want a bitmap greater than 256x256 you will need more
	// memory so another vram bank must be used and mapped consecutivly
	vramSetMainBanks(	VRAM_A_MAIN_BG_0x6000000, VRAM_B_LCD,
				VRAM_C_SUB_BG, VRAM_D_LCD);
	lcdMainOnBottom();

	if(DEBUG) {
		// set the sub background up for text display (we could just print to one
		// of the main display text backgrounds just as easily
		videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE); //sub bg 0 will be used to print text

		// set up text background for text
		SUB_BG0_CR = BG_MAP_BASE(31);

		BG_PALETTE_SUB[255] = RGB15(31,31,31);//by default font will be rendered with color 255

		//consoleInit() is a lot more flexible but this gets you up and running quick
		consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);

		splash_fade=-1;
	} else {
		videoSetModeSub(MODE_5_2D | DISPLAY_BG3_ACTIVE);
		// set up our bitmap background

		SUB_BG3_CR = BG_BMP16_256x256;

		// these are rotation backgrounds so you must set the rotation attributes:
		// these are fixed point numbers with the low 8 bits the fractional part
		// this basicaly gives it a 1:1 translation in x and y so you get a nice flat bitmap
		SUB_BG3_XDX = 1 << 8;
		SUB_BG3_XDY = 0;
		SUB_BG3_YDX = 0;
		SUB_BG3_YDY = 1 << 8;

		SUB_BG3_CX = 0;
		SUB_BG3_CY = 0;

		sImage pcx;

		//load our texture
		loadPCX((u8*)splash_pcx, &pcx);

		image8to16(&pcx);
		
		swiWaitForVBlank();
		SUB_BLEND_CR = BLEND_FADE_BLACK | BLEND_SRC_BG3;
		SUB_BLEND_Y = 31;

		int x,y;

		for(y=0; y<192; y++)
			for(x=0; x<256; x++)
				TopScreen[x+y*256] = pcx.data16[x+y*256] | BIT(15);

		imageDestroy(&pcx);
	}

	// set up our bitmap background

	BG3_CR = BG_BMP16_256x256;

	// these are rotation backgrounds so you must set the rotation attributes:
	// these are fixed point numbers with the low 8 bits the fractional part
	// this basicaly gives it a 1:1 translation in x and y so you get a nice flat bitmap
	BG3_XDX = 1 << 8;
	BG3_XDY = 0;
	BG3_YDX = 0;
	BG3_YDY = 1 << 8;

	BG3_CX = 0;
	BG3_CY = 0;

	GenerateSkin();
	lcdMainOnBottom();

	fatInitDefault();
	
	Do_LoadConfig((void *)1);

	if(!NO3D) {
		powerON(POWER_ALL); 
	}

	irqSet(IRQ_FIFO_NOT_EMPTY, __fifoIRQ);
//	if(DEBUG) iprintf("irqSet(IRQ_FIFO_NOT_EMPTY, __fifoIRQ);\n");
	irqEnable(IRQ_FIFO_NOT_EMPTY);  
      
//	if(DEBUG) iprintf("irqEnable(IRQ_FIFO_NOT_EMPTY);\n");
   
	REG_IPC_FIFO_CR = IPC_FIFO_ENABLE | IPC_FIFO_SEND_CLEAR | IPC_FIFO_RECV_IRQ;
//	if(DEBUG) iprintf("REG_IPC_FIFO_CR Set\n");

//	if(DEBUG) iprintf("Wifi_Init();\n");
	u32 Wifi_pass= Wifi_Init(WIFIINIT_OPTION_USELED);
//	if(DEBUG) iprintf("fifo_WiFiInit(0x%08x);\n", Wifi_pass);
	fifo_WiFiInit(Wifi_pass);

//	if(DEBUG) iprintf("irqSet(IRQ_TIMER3, Timer_50ms);\n");

	irqSet(IRQ_TIMER3, Timer_50ms);

//	if(DEBUG) iprintf("irqEnable(IRQ_TIMER3);\n");
	irqEnable(IRQ_TIMER3);

//	if(DEBUG) iprintf("Wifi_SetSyncHandler();\n");
	Wifi_SetSyncHandler(fifo_WiFiSync);

	*((volatile u16 *)0x0400010C) = -6553; // 6553.1 * 256 cycles = ~50ms;
	*((volatile u16 *)0x0400010E) = 0x00C2; // enable, irq, 1/256 clock

//	if(DEBUG) iprintf("Wifi_CheckInit();\n");
	while(Wifi_CheckInit()==0) WaitVbl();

	memset(BackBuffer, 0, sizeof(BackBuffer));

//	if(DEBUG) iprintf("Do_Menu();\n");

	Do_Menu(&MainMenu);
	
	Do_PowerOff(NULL);

	return 0;
}
