// MythRemote - copyright David Kuder (GrizzlyAdams) 2007
// Based on DSWifi Test Project - copyright Stephen Stair 2005
// Based on template project in ndslib by wntrmute
/****************************************************************************** 
DSWifi Lib and test materials are licenced under the MIT open source licence:
Copyright (c) 2005 Stephen Stair

MythRemote is also licensed under the MIT open source license:
Copyright (c) 2007 David Kuder

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
******************************************************************************/


#include "nds.h"
#include <nds/arm9/console.h>		//basic print funcionality
#include <nds/arm9/image.h>		//needed to load pcx files


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>

#include <fat.h>
#include <cartreset.h>

#include "mythremote.h"

// Redefined this in the wifi libs so we need to deal with that.
#undef FD_SETSIZE

#include <dswifi9.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>




Button_t IPConfigButtons[] = {
 {   4,   4, 56, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "IP1", SelectBox, (void *)0, 0 },
 {  68,   4, 56, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "IP2", SelectBox, (void *)1, 0 },
 { 132,   4, 56, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "IP3", SelectBox, (void *)2, 0 },
 { 196,   4, 56, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "IP4", SelectBox, (void *)3, 0 },

 {  72,  28, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "1", PushIPDigit, (void *)1, 0 },
 { 112,  28, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "2", PushIPDigit, (void *)2, 0 },
 { 152,  28, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "3", PushIPDigit, (void *)3, 0 },
 {  72,  68, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "4", PushIPDigit, (void *)4, 0 },
 { 112,  68, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "5", PushIPDigit, (void *)5, 0 },
 { 152,  68, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "6", PushIPDigit, (void *)6, 0 },
 {  72, 108, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "7", PushIPDigit, (void *)7, 0 },
 { 112, 108, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "8", PushIPDigit, (void *)8, 0 },
 { 152, 108, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "9", PushIPDigit, (void *)9, 0 },

 {  72, 148, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "<", PopIPDigit,  (void *)0, 0 },
 { 112, 148, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "0", PushIPDigit, (void *)0, 0 },
 { 152, 148, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "C", PopIPDigit,  (void *)1, 0 },

 {   4, 164, 56, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "Back", SelectBox, (void *)-1, Effect_Up },
 { 196, 164, 56, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "Next", SelectBox, (void *)-2, Effect_Right },

 { 0, 0, 0, 0, libskin_BC_Invalid, 0, 0, NULL, NULL, NULL, 0 }
};

ButtonPanel_t IPConfigMenu = {
	1,	// Accept Cursor Input
	0,	// Select 1st entry
	-1,	// TouchState
	0,	// Frames
	0,	// Bool Exit
	Effect_Up,
	IPConfigButtons
};

Button_t PortConfig_Buttons[] = {
 { 72, 4, 112, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "Port:      ", NULL, NULL, 0 },

 {  72,  28, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "1", PushPortDigit, (void *)1, 0 },
 { 112,  28, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "2", PushPortDigit, (void *)2, 0 },
 { 152,  28, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "3", PushPortDigit, (void *)3, 0 },
 {  72,  68, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "4", PushPortDigit, (void *)4, 0 },
 { 112,  68, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "5", PushPortDigit, (void *)5, 0 },
 { 152,  68, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "6", PushPortDigit, (void *)6, 0 },
 {  72, 108, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "7", PushPortDigit, (void *)7, 0 },
 { 112, 108, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "8", PushPortDigit, (void *)8, 0 },
 { 152, 108, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "9", PushPortDigit, (void *)9, 0 },

 {  72, 148, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "<", PopPortDigit,  (void *)0, 0 },
 { 112, 148, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "0", PushPortDigit, (void *)0, 0 },
 { 152, 148, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "C", PopPortDigit,  (void *)1, 0 },

 {   4, 164, 56, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "Back", SelectBox, (void *)-2, Effect_Left },
 { 196, 164, 56, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "Done", SelectBox, (void *)-1, Effect_Up },

 { 0, 0, 0, 0, libskin_BC_Invalid, 0, 0, NULL, NULL, NULL, 0 }
};

ButtonPanel_t PortConfigMenu = {
	1,	// Accept Cursor Input
	0,	// Select 1st entry
	-1,	// TouchState
	0,	// Frames
	0,	// Bool Exit
	Effect_Up,
	PortConfig_Buttons
};

void ExitPanel(void *arg) {
	ButtonPanel_t *pPanel=(ButtonPanel_t *)arg;
	pPanel->Exit=1;
}

ButtonPanel_t ConfigMenu;
#define CFG_NO3D 3

void SwitchNO3D(void *arg) {
	NO3D=!NO3D;
	ConfigMenu.ButtonList[CFG_NO3D].Checked = !NO3D;

	swiWaitForVBlank();
	DrawButton(&Skin, &ConfigMenu.ButtonList[CFG_NO3D], BottomScreen);
}

Button_t ConfigMenu_Buttons[] = {
 { 28,12,200,16, libskin_BC_Normal, libskin_BS_Normal, 0, "MythFrontend 1", Do_IPConfig, &Host[0], 0 },
 { 28,36,200,16, libskin_BC_Normal, libskin_BS_Normal, 0, "MythFrontend 2", Do_IPConfig, &Host[1], 0 },
 { 28,60,200,16, libskin_BC_Normal, libskin_BS_Normal, 0, "MythFrontend 3", Do_IPConfig, &Host[2], 0 },

 { 28,84,200,16, libskin_BC_CheckBox, libskin_BS_Normal, 0, "3D Effects", SwitchNO3D, NULL, 0 },

 { 28,108,88,16, libskin_BC_Normal, libskin_BS_Normal, 0, "Load", Do_LoadConfig, NULL, Effect_Fade },
 { 140,108,88,16, libskin_BC_Normal, libskin_BS_Normal, 0, "Save", Do_SaveConfig, NULL, Effect_Fade },

 { 100, 160, 56, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "Exit", ExitPanel, &ConfigMenu, 0 },

 { 0, 0, 0, 0, libskin_BC_Invalid, 0, 0, NULL, NULL, NULL, 0 }
};

ButtonPanel_t ConfigMenu = {
	1,	// Accept Cursor Input
	0,	// Select 1st entry
	-1,	// TouchState
	0,	// Frames
	0,	// Bool Exit
	Effect_Down,
	ConfigMenu_Buttons
};


int IPDigit[4]= {192,168,0,1};

int SelectedBox=0;
int PortDigits=0;
int SaveDigits=0;

void SelectBox(void *arg) {
	int x = (int)arg;
	SelectedBox=x;
}

void PushIPDigit(void *arg) {
	int x = (int)arg;
	IPDigit[SelectedBox] *= 10;
	IPDigit[SelectedBox] += x;
	if(IPDigit[SelectedBox]>255) IPDigit[SelectedBox] %= 100;;
	SaveDigits=1;
}

void PopIPDigit(void *arg) {
	if(arg==NULL) {
		IPDigit[SelectedBox] /= 10;
	} else {
		IPDigit[SelectedBox] = 0;
	}
	SaveDigits=1;
}

void Do_IPConfig(void *arg) {
	Host_t *Frontend = (Host_t *)arg;
	ButtonPanel_t *pPanel = &IPConfigMenu;
	Button_t *pButtonList = pPanel->ButtonList;
	Button_t *pSelected = NULL;
	int i;

	pPanel->Exit=0;
	if(pPanel->CursorInput)
		pPanel->Selected=0;
	else
		pPanel->Selected=-1;
	pPanel->TouchState=-1;

	IPDigit[0] = ((Frontend->IP)>>24) & 0xff;
	IPDigit[1] = ((Frontend->IP)>>16) & 0xff;
	IPDigit[2] = ((Frontend->IP)>>8) & 0xff;
	IPDigit[3] = (Frontend->IP) & 0xff;
	
	for(i=0; i<4; i++)
		sprintf(IPConfigButtons[i].Label, "%3i", IPDigit[i]);

	SelectedBox=0;

	Transition(Effect_Up | Effect_Out, &ConfigMenu);
	Transition(Effect_Up | Effect_In, pPanel);

	while(!pPanel->Exit) {
		if(SaveDigits) {
			sprintf(IPConfigButtons[SelectedBox].Label, "%3i", IPDigit[SelectedBox]);
			SaveDigits=0;
		}
		
		for(i=0; i<4; i++) {
			if(i==SelectedBox) IPConfigButtons[i].State|=libskin_BS_Pressed;
			else IPConfigButtons[i].State&=~libskin_BS_Pressed;
		}

		DrawBackdrop(&Skin, BackBuffer);
		RenderMenu(pPanel, BackBuffer);

		WaitVbl();

		dmaCopy(BackBuffer, BottomScreen, 256*192*2);

		while(!pPanel->Exit) {
			WaitVbl();
			
			if(InputMenu(pPanel)) break;
		}
		while(keysHeld()&(KEY_TOUCH | KEY_A | KEY_B | KEY_X | KEY_Y))
			WaitVbl();

		pPanel->Frames=0;
		pSelected = &pButtonList[pPanel->Selected];
		if(pSelected->Function!=NULL)
			pSelected->Function(pSelected->Argument);

		if(SelectedBox==-2) {
			Frontend->IP = (IPDigit[0]<<24) | (IPDigit[1]<<16) | (IPDigit[2]<<8) | IPDigit[3];
			Transition(Effect_Right | Effect_Out, pPanel);
			Do_PortConfig(Frontend);
			if(SelectedBox!=-1) {
				Transition(Effect_Left | Effect_In, pPanel);
			} else {
				return;
			}
		}

		if(SelectedBox==-1) {
			pPanel->Exit=1;
			Transition(Effect_Down | Effect_Out, pPanel);
			Transition(Effect_Down | Effect_In, &ConfigMenu);
			return;
		}
	}

	SelectedBox=0;
}


void PushPortDigit(void *arg) {
	int x = (int)arg;
	PortDigits *= 10;
	if(PortDigits>65535)
		PortDigits %= 10000;
	PortDigits += x;
	SaveDigits=1;
}

void PopPortDigit(void *arg) {
	if(arg==NULL) {
		PortDigits /= 10;
	} else {
		PortDigits = 0;
	}
	SaveDigits=1;
}


void Do_PortConfig(void *arg) {
	Host_t *Frontend = (Host_t *)arg;
	ButtonPanel_t *pPanel = &PortConfigMenu;
	Button_t *pButtonList = pPanel->ButtonList;
	Button_t *pSelected = NULL;

	pPanel->Exit=0;
	if(pPanel->CursorInput)
		pPanel->Selected=0;
	else
		pPanel->Selected=-1;
	pPanel->TouchState=-1;

	PortDigits = Frontend->Port;
	
	SelectedBox=0;
	sprintf(pButtonList[0].Label, "Port: %5i", PortDigits);

	Transition(Effect_Right | Effect_In, pPanel);

	while(!pPanel->Exit) {
		if(SaveDigits) {
			sprintf(pButtonList[0].Label, "Port: %5i", PortDigits);
			Frontend->Port=PortDigits;
			SaveDigits=0;
		}

		pButtonList[0].State|=2;

		DrawBackdrop(&Skin, BackBuffer);
		RenderMenu(pPanel, BackBuffer);

		WaitVbl();

		dmaCopy(BackBuffer, BottomScreen, 256*192*2);

		while(!pPanel->Exit) {
			WaitVbl();
			
			if(InputMenu(pPanel)) break;
		}
		while(keysHeld()&(KEY_TOUCH | KEY_A | KEY_B | KEY_X | KEY_Y))
			WaitVbl();

		pPanel->Frames=0;
		pSelected = &pButtonList[pPanel->Selected];
		if(pSelected->Function!=NULL)
			pSelected->Function(pSelected->Argument);

		if(SelectedBox==-1) {
			pPanel->Exit=1;
			SelectedBox=-1;
			Frontend->Port = PortDigits;
			Transition(Effect_Down | Effect_Out, pPanel);
			Transition(Effect_Down | Effect_In, &ConfigMenu);
		}
		if(SelectedBox==-2) {
			pPanel->Exit=1;
			SelectedBox=0;
			Transition(Effect_Left | Effect_Out, pPanel);
		}
	}
}

char CFG_MAGIC[] = "MythRemoteConfig";
unsigned long CFG_VERS = 0x1100;

void Do_SaveConfig(void *arg) {
	FILE *of;

	of=fopen("fat0:/MythRemote/config.bin", "w");
	if(of==NULL) {
		Do_Dialog("Unable to open config file.");
		return;
	}

	fwrite(CFG_MAGIC, sizeof(CFG_MAGIC), 1, of);
	fwrite(&CFG_VERS, 4, 1, of);
	fwrite(&Host[0].IP, 4, 1, of);
	fwrite(&Host[0].Port, 2, 1, of);
	fwrite(&Host[1].IP, 4, 1, of);
	fwrite(&Host[1].Port, 2, 1, of);
	fwrite(&Host[2].IP, 4, 1, of);
	fwrite(&Host[2].Port, 2, 1, of);
	fwrite(&NO3D, sizeof(NO3D), 1, of);

	fclose(of);

	Do_Dialog("Config Saved.");
}

void Do_LoadConfig(void *arg) {
	FILE *of;
	char magic[sizeof(CFG_MAGIC)];
	int noerror=1;
	unsigned long vers;
	int silent = (int)arg;

	of=fopen("fat0:/MythRemote/config.bin", "r");
	if(of==NULL) {
		if(!silent)
			Do_Dialog("Unable to open config file.");
		return;
	}

	fread(magic, sizeof(magic), 1, of);
	if(!strcmp(magic, CFG_MAGIC)) {
		fread(&vers, 4, 1, of);
		if(vers==CFG_VERS) {
			fread(&Host[0].IP, 4, 1, of);
			fread(&Host[0].Port, 2, 1, of);
			fread(&Host[1].IP, 4, 1, of);
			fread(&Host[1].Port, 2, 1, of);
			fread(&Host[2].IP, 4, 1, of);
			fread(&Host[2].Port, 2, 1, of);
			fread(&NO3D, sizeof(NO3D), 1, of);
		} else if(vers==0x1000) {
			fread(&Host[0].IP, 4, 1, of);
			fread(&Host[1].IP, 4, 1, of);
			fread(&Host[2].IP, 4, 1, of);
			fread(&Host[0].Port, 2, 1, of);
			Host[2].Port=Host[1].Port=Host[0].Port;
		} else {
			Do_Dialog("Incompatible config version.");
			noerror=0;
		}
	} else {
		Do_Dialog("Corrupt config file.");
		noerror=0;
	}

	fclose(of);

	if(noerror && !silent)
		Do_Dialog("Config Loaded.");

}



void Do_Config(void *arg) {
	ConfigMenu.ButtonList[CFG_NO3D].Checked = !NO3D;
	Do_Menu(&ConfigMenu);
}
