// MythRemote - copyright David Kuder (GrizzlyAdams) 2007
// Based on DSWifi Test Project - copyright Stephen Stair 2005
// Based on template project in ndslib by wntrmute
/****************************************************************************** 
DSWifi Lib and test materials are licenced under the MIT open source licence:
Copyright (c) 2005 Stephen Stair

MythRemote is also licensed under the MIT open source license:
Copyright (c) 2007 David Kuder

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
******************************************************************************/


#include "nds.h"
#include <nds/arm9/console.h>		//basic print funcionality
#include <nds/arm9/image.h>		//needed to load pcx files


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>

#include <fat.h>
#include <cartreset.h>

#include "mythremote.h"

// Redefined this in the wifi libs so we need to deal with that.
#undef FD_SETSIZE

#include <dswifi9.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>




void GenerateSkin(void) {
	int x,y,z;
	for(y=0; y<192; y++) 
		for(x=0; x<256; x++)
			Skin.Backdrop[y*256+x]=RGB(0,0,0) | BIT(15);

	for(y=0; y<32; y++) 
		for(x=0; x<256; x++)
			Skin.Button.Image[0][y*256+x]=RGB(((31-y)/2+8),((31-y)/2+8),((31-y)/2+8)) | BIT(15);
	for(y=0; y<32; y++) 
		for(x=0; x<256; x++)
			Skin.Button.Image[1][y*256+x]=RGB((31-y),0,0) | BIT(15);
	for(y=0; y<32; y++) 
		for(x=0; x<256; x++)
			Skin.Button.Image[2][y*256+x]=RGB(0,(31-y),0) | BIT(15);
	for(y=0; y<32; y++) 
		for(x=0; x<256; x++)
			Skin.Button.Image[3][y*256+x]=RGB(0,0,(31-y)) | BIT(15);

	Skin.Button.AlignX=libskin_IAX_Center;
	Skin.Button.AlignY=libskin_IAY_Center;
	Skin.Button.Highlight=RGB(31,31,31)|BIT(15);
	Skin.Button.Shadow=RGB(0,0,0)|BIT(15);
	Skin.TextColor=RGB(0,0,0)|BIT(15);


	sImage pcx;

	//load our texture
	loadPCX((u8*)check_pcx, &pcx);

	image8to16(&pcx);

	unsigned short *checkimg = pcx.data16;
	unsigned short Col;

	for(z=0; z<2; z++)
		for(y=0; y<12; y++)
			for(x=0; x<12; x++) {
				Col = *checkimg++;
				Skin.Check[z][y*12+x] = (Col&0x3e0)?Col:0x0000;
			}

	for(z=0; z<2; z++)
		for(y=0; y<12; y++)
			for(x=0; x<12; x++) {
				Col = *checkimg++;
				Skin.Radio[z][y*12+x] = (Col&0x3e0)?Col:0x0000;
			}


	imageDestroy(&pcx);

	memcpy(Skin.Font, ASCII_bin, 256*8*8);
}


void UpdateMenu(ButtonPanel_t *pPanel, int NewSelected) {
	Button_t *pButtonList = pPanel->ButtonList;
	int i;

	if(pPanel->Selected == NewSelected) return;

	i=pPanel->Selected;
	if(i>=0) {
		pButtonList[i].State &= ~libskin_BS_Highlight;
		DrawButton(&Skin, &pButtonList[i], BottomScreen);
	}

	i=NewSelected;
	if(i>=0) {
		pButtonList[i].State |= libskin_BS_Highlight;
		DrawButton(&Skin, &pButtonList[i], BottomScreen);
	}

	pPanel->Selected = NewSelected;
}



void RenderMenu(ButtonPanel_t *pPanel, unsigned short *Screen) {

	DrawButtonPanel(&Skin, pPanel, Screen);

	if(pPanel->CursorInput) {
		BlitCenterText(&Skin, 184, "[UP/DN]Move[A/Touch]Sel", Skin.Button.Highlight, Screen);
	}
}

void Display2D(int Black) {
	int i;
	if(Black)
		for(i=0; i<256*192; i++) BackBuffer[32*256+i]=0x8000;

	WaitVbl();
	if(DEBUG) iprintf("Display2D()\n");
	videoSetMode(MODE_5_2D | DISPLAY_BG3_ACTIVE);
	vramSetMainBanks(	VRAM_A_MAIN_BG_0x6000000, VRAM_B_LCD,
				VRAM_C_SUB_BG, VRAM_D_LCD);

	BG3_CR = BG_BMP16_256x256;

	// these are rotation backgrounds so you must set the rotation attributes:
	// these are fixed point numbers with the low 8 bits the fractional part
	// this basicaly gives it a 1:1 translation in x and y so you get a nice flat bitmap
	BG3_XDX = 1 << 8;
	BG3_XDY = 0;
	BG3_YDX = 0;
	BG3_YDY = 1 << 8;

	BG3_CX = 0;
	BG3_CY = 0;

	dmaCopy(&BackBuffer[32*256], BottomScreen, 256*192*2);
}

int AntiEffect(int Effect) {
	int OutEffect = Effect & 0xf0;
	if(Effect & Effect_Up) OutEffect |= Effect_Down;
	if(Effect & Effect_Down) OutEffect |= Effect_Up;
	if(Effect & Effect_Left) OutEffect |= Effect_Right;
	if(Effect & Effect_Right) OutEffect |= Effect_Left;
	return OutEffect;
}

void Transition(int Effect, void *param) {
	int i;
	ButtonPanel_t *pPanel = (ButtonPanel_t *)param;

	if(param == NULL) return;

	if(!(Effect & Effect_In)&&(splash_fade>30))
			splash_fade=30;

	if(((Effect & 0xf) == 0) || NO3D) {
		WaitVbl();
		if(Effect & Effect_In) {
			BLEND_Y=31;
		} else {
			BLEND_Y=0;
		}

		BLEND_CR=BLEND_FADE_BLACK | BLEND_SRC_BG3;

		DrawBackdrop(&Skin, &BackBuffer[32*256]);
		RenderMenu(pPanel, &BackBuffer[32*256]);
		dmaCopy(&BackBuffer[32*256], BottomScreen, 256*192*2);

		if(Effect & Effect_In) {
			for(i=31; i>0; i--) {
				WaitVbl();
				BLEND_Y=i;
			}
			WaitVbl();
			BLEND_Y=0;
		} else {
			for(i=0; i<32; i++) {
				WaitVbl();
				BLEND_Y=i;
			}
			WaitVbl();
			BLEND_Y=31;
			for(i=0; i<256*192; i++) BottomScreen[i]=0x8000;
			WaitVbl();
			BLEND_CR=BLEND_NONE;
		}
	} else {
		DrawBackdrop(&Skin, &BackBuffer[32*256]);
		RenderMenu(pPanel, &BackBuffer[32*256]);

		Do3DEffect(Effect, BackBuffer);
		Display2D((Effect & Effect_In) ? 0 : 1);
	}
}


int InputMenu(ButtonPanel_t *pPanel) {
	Button_t *pButtonList = pPanel->ButtonList;
	int state = pPanel->Selected;
	int i;

	if(pPanel->CursorInput) {
		if(keysDown()&~(KEY_LID | KEY_TOUCH))
			pPanel->Frames=0;

		if(keysDown()&KEY_UP) {
			if(state==0) while(pButtonList[state+1].Class) state++;
			else state--;
		}
		if(keysDown()&KEY_DOWN) {
			state++;
			if(!pButtonList[state].Class) state=0;
		}
		if(keysDown()&(KEY_A | KEY_START)) {
			return 1; // state confirmed
		}
		if((keysDown()&~KEY_LID) && !screen_on) {
			LCD_FadeIn(pPanel);
			screen_on=1;
			pPanel->Frames=0;
			while(keysHeld()&~KEY_LID) WaitVbl();
		}
	}
	
	if((keysUp()&KEY_LID) && !screen_on) {
		LCD_FadeIn(pPanel);
		screen_on=1;
		pPanel->Frames=0;
	}
	if((keysDown()&KEY_TOUCH) && !screen_on) {
		LCD_FadeIn(pPanel);
		screen_on=1;
		pPanel->Frames=0;
		while(keysHeld()&KEY_TOUCH) WaitVbl();
	}
	if(keysUp()&KEY_TOUCH && pPanel->TouchState==state) { // just released
		pPanel->Frames=0;
		return 1; // state confirmed.
	}
	pPanel->TouchState=-1;
	if(pPanel->Frames>5) {
		if(keysHeld()&KEY_TOUCH) { // update pPanel->TouchState
			for(i=0; pButtonList[i].Class; i++) {
				if((touchXY.px>=pButtonList[i].X) && (touchXY.px<(pButtonList[i].X+pButtonList[i].W))
						&& (touchXY.py>=pButtonList[i].Y) && (touchXY.py<(pButtonList[i].Y+pButtonList[i].H)) ) {
					pPanel->TouchState=i;
					pPanel->Frames=15;
				}
			}
			if(pPanel->TouchState!=-1) state=pPanel->TouchState;
		}
	} else {
		if(keysHeld()&KEY_TOUCH)
			pPanel->Frames=0;
	}
	pPanel->Frames++;

			
	if(pPanel->Frames<100 && !screen_on && !(keysHeld()&KEY_LID)) {
		LCD_FadeIn(pPanel);
		screen_on=1;
		pPanel->Frames=0;
	}

	if(((pPanel->Frames > 1800) || (keysDown()&KEY_LID)) && screen_on) {
		// Turn screens off.
		LCD_FadeOut(pPanel);
		screen_on=0;
	}

	if(state!=pPanel->Selected)
		UpdateMenu(pPanel, state);

	return 0;
}

void Do_Menu(void *param) {
	ButtonPanel_t *pPanel = (ButtonPanel_t *)param;
	Button_t *pButtonList = pPanel->ButtonList;
	Button_t *pSelected = NULL;

	pPanel->Exit=0;
	if(pPanel->CursorInput)
		pPanel->Selected=0;
	else
		pPanel->Selected=-1;
	pPanel->TouchState=-1;


	Transition(pPanel->Transition | Effect_In, pPanel);

	while(!pPanel->Exit) {
		DrawBackdrop(&Skin, BackBuffer);
		RenderMenu(pPanel, BackBuffer);

		WaitVbl();
		dmaCopy(BackBuffer, BottomScreen, 256*192*2);

		while(!pPanel->Exit) {
			WaitVbl();
			
			if(InputMenu(pPanel)) break;
		}
		while(keysHeld()&~KEY_LID)
			WaitVbl();

		pPanel->Frames=0;
		if(pPanel->Selected>=0) {
			pSelected = &pButtonList[pPanel->Selected];
			if(pSelected->Transition) {
				Transition(pSelected->Transition | Effect_Out, pPanel);
			}
			if(pSelected->Function!=NULL)
				pSelected->Function(pSelected->Argument);
			if(pSelected->Transition) {
				Transition(AntiEffect(pSelected->Transition) | Effect_In, pPanel);
			}
		}

		while(keysHeld()&~KEY_LID)
			WaitVbl();
	}
	Transition(AntiEffect(pPanel->Transition) | Effect_Out, pPanel);
}


Button_t DialogButtons[] = {
	{ 84, 120, 80, 24, libskin_BC_Normal, libskin_BS_Normal, 0, "OK", CloseDialog, NULL, 0 },
	{ 0, 0, 0, 0, libskin_BC_Invalid, 0, 0, NULL, NULL, NULL, 0 }
};

ButtonPanel_t DialogPanel = {
	1,	// Accept Cursor Input
	0,	// Select 1st entry
	-1,	// TouchState
	0,	// Frames
	0,	// Bool Exit
	Effect_Fade,
	DialogButtons
};

void CloseDialog(void *arg) {
	DialogPanel.Exit = 1;
}

void Do_Dialog(char *message) {
	ButtonPanel_t *pPanel = &DialogPanel;
	Button_t *pButtonList = pPanel->ButtonList;
	Button_t *pSelected = NULL;

	pPanel->Exit=0;
	if(pPanel->CursorInput)
		pPanel->Selected=0;
	else
		pPanel->Selected=-1;
	pPanel->TouchState=-1;


	Transition(pPanel->Transition | Effect_In, pPanel);

	while(!pPanel->Exit) {
		DrawBackdrop(&Skin, BackBuffer);
		RenderMenu(pPanel, BackBuffer);
		
		if(message)
			BlitCenterText(&Skin, 64, message, Skin.Button.Highlight, BackBuffer);

		WaitVbl();
		dmaCopy(BackBuffer, BottomScreen, 256*192*2);

		while(!pPanel->Exit) {
			WaitVbl();
			
			if(InputMenu(pPanel)) break;
		}
		while(keysHeld()&~KEY_LID)
			WaitVbl();

		pPanel->Frames=0;
		if(pPanel->Selected>=0) {
			pSelected = &pButtonList[pPanel->Selected];
			if(pSelected->Transition) {
				Transition(pSelected->Transition | Effect_Out, pPanel);
			}
			if(pSelected->Function!=NULL)
				pSelected->Function(pSelected->Argument);
			if(pSelected->Transition) {
				Transition(AntiEffect(pSelected->Transition) | Effect_In, pPanel);
			}
		}

		while(keysHeld()&~KEY_LID)
			WaitVbl();
	}
	Transition(AntiEffect(pPanel->Transition) | Effect_Out, pPanel);
}
