// MythRemote - copyright David Kuder (GrizzlyAdams) 2007
// Based on DSWifi Test Project - copyright Stephen Stair 2005
// Based on template project in ndslib by wntrmute
/****************************************************************************** 
DSWifi Lib and test materials are licenced under the MIT open source licence:
Copyright (c) 2005 Stephen Stair

MythRemote is also licensed under the MIT open source license:
Copyright (c) 2007 David Kuder

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
******************************************************************************/


#include "nds.h"
#include <nds/arm9/console.h>		//basic print funcionality
#include <nds/arm9/image.h>		//needed to load pcx files


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>

#include <fat.h>
#include <cartreset.h>

#include "mythremote.h"
#include "tcpstate.h"

// Redefined this in the wifi libs so we need to deal with that.
#undef FD_SETSIZE

#include <dswifi9.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>


#define REMOTEPAGES 3
#define MAXBUTTONS 32
enum cmd_numbers {
	CMD_0=0, CMD_1, CMD_2, CMD_3, CMD_4,
	CMD_5, CMD_6, CMD_7, CMD_8, CMD_9,
	CMD_UP, CMD_DOWN, CMD_LEFT, CMD_RIGHT,
	CMD_SELECT, CMD_BACK, CMD_ENTER,
	CMD_PAGEUP, CMD_PAGEDOWN,
	CMD_SKIP, CMD_MENU, CMD_INFO,
	CMD_VOLUP, CMD_VOLDOWN, CMD_MUTE,
	CMD_RECORD,
	CMD_EXIT,
	
	JUMP_MAINMENU, JUMP_LIVETV, JUMP_MYTHGALLERY,
	JUMP_PLAYMUSIC, JUMP_RECORDINGS, JUMP_VIDEOGALLERY,
	JUMP_PLAYDVD,
	
	PLAY_PAUSE, PLAY_PLAY, PLAY_STOP,
	PLAY_CHUP, PLAY_CHDOWN,
	
	PLAY_REV, PLAY_2X, PLAY_4X, PLAY_16X,
	PLAY_1_16X, PLAY_1_8X, PLAY_1_4X, PLAY_1_2X,
	PLAY_BEGINNING, PLAY_BACK, PLAY_FWD,
	
	CMD_MAX
};

char CommandText[CMD_MAX][32] = {
	"key 0\n",	"key 1\n",	"key 2\n",	"key 3\n",	"key 4\n",
	"key 5\n",	"key 6\n",	"key 7\n",	"key 8\n",	"key 9\n",
	"key up\n",	"key down\n",	"key left\n",	"key right\n",
	"key space\n",	"key escape\n",	"key enter\n",
	"key pageup\n",	"key pagedown\n",
	"key z\n",	"key m\n",	"key i\n",
	"key bracketright\n",	"key bracketleft\n",	"key shift-bar\n",
	"key r\n",
	"key exit\n",
	
	"jump mainmenu\n",
	"jump livetv\n",
	"jump mythgallery\n",
	"jump playmusic\n",
	"jump playbackrecordings\n",
	"jump videogallery\n",
	"jump playdvd\n",

	"play speed pause\n", "play speed normal\n",
	"play stop\n",
	"play channel up\n", "play channel down\n",

	"play speed -1x\n", "play speed 2x\n",
	"play speed 4x\n", "play speed 16x\n",

	"play speed 1/16x\n", "play speed 1/8x\n",
	"play speed 1/4x\n", "play speed 1/2x\n",

	"play seek beginning\n", "play seek backward\n",
	"play seek forward\n",
};


Button_t TouchButtons[REMOTEPAGES][MAXBUTTONS] = {
{
 { 4, 164, 80, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "<- 3", PrevScreen, NULL, Effect_Left },
 { 88, 164, 80, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "Exit", DisconnectFrontend, (void*)1, 1 },
 { 172, 164, 80, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "2 ->", NextScreen, NULL, Effect_Right },

 { 140, 124, 72, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "Enter", SendCommand, CommandText[CMD_ENTER], 0 },
 { 220, 124, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "0", SendCommand, CommandText[CMD_0], 0 },
 { 140, 4, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "1", SendCommand, CommandText[CMD_1], 0 },
 { 180, 4, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "2", SendCommand, CommandText[CMD_2], 0 },
 { 220, 4, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "3", SendCommand, CommandText[CMD_3], 0 },
 { 140, 44, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "4", SendCommand, CommandText[CMD_4], 0 },
 { 180, 44, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "5", SendCommand, CommandText[CMD_5], 0 },
 { 220, 44, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "6", SendCommand, CommandText[CMD_6], 0 },
 { 140, 84, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "7", SendCommand, CommandText[CMD_7], 0 },
 { 180, 84, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "8", SendCommand, CommandText[CMD_8], 0 },
 { 220, 84, 32, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "9", SendCommand, CommandText[CMD_9], 0 },

 { 68, 4, 56, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "V+", SendCommand, CommandText[CMD_VOLUP], 0 },
 { 68, 44, 56, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "Mute", SendCommand, CommandText[CMD_MUTE], 0 },
 { 68, 84, 56, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "V-", SendCommand, CommandText[CMD_VOLDOWN], 0 },

 { 4, 4, 56, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "CH +", SendCommand, CommandText[PLAY_CHUP], 0 },
 { 4, 44, 56, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "Info", SendCommand, CommandText[CMD_INFO], 0 },
 { 4, 84, 56, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "CH -", SendCommand, CommandText[PLAY_CHDOWN], 0 },

 { 4, 124, 56, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "Menu", SendCommand, CommandText[CMD_MENU], 0 },
 { 68, 124, 56, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "Back", SendCommand, CommandText[CMD_BACK], 0 },

 { 0, 0, 0, 0, libskin_BC_Invalid, 0, 0, NULL, NULL, NULL, 0 }
},
{
 { 4, 164, 80, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "<- 1", PrevScreen, NULL, Effect_Left },
 { 88, 164, 80, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "Exit", DisconnectFrontend, (void*)1, 1 },
 { 172, 164, 80, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "3 ->", NextScreen, NULL, Effect_Right },

 { 4, 4, 80, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "Pause", SendCommand, CommandText[PLAY_PAUSE], 0 },
 { 88, 4, 80, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "Play", SendCommand, CommandText[PLAY_PLAY], 0 },
 { 172, 4, 80, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "Stop", SendCommand, CommandText[PLAY_STOP], 0 },

 { 4, 44, 56, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "-1x", SendCommand, CommandText[PLAY_REV], 0 },
 { 68, 44, 56, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "2x", SendCommand, CommandText[PLAY_2X], 0 },
 { 132, 44, 56, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "4x", SendCommand, CommandText[PLAY_4X], 0 },
 { 196, 44, 56, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "16x", SendCommand, CommandText[PLAY_16X], 0 },

 { 4, 84, 56, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "1/16", SendCommand, CommandText[PLAY_1_16X], 0 },
 { 68, 84, 56, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "1/8", SendCommand, CommandText[PLAY_1_8X], 0 },
 { 132, 84, 56, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "1/4", SendCommand, CommandText[PLAY_1_4X], 0 },
 { 196, 84, 56, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "1/2", SendCommand, CommandText[PLAY_1_2X], 0 },

 { 4, 124, 80, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "Beginning", SendCommand, CommandText[PLAY_BEGINNING], 0 },
 { 88, 124, 80, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "Back", SendCommand, CommandText[PLAY_BACK], 0 },
 { 172, 124, 80, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "Forward", SendCommand, CommandText[PLAY_FWD], 0 },

 { 0, 0, 0, 0, libskin_BC_Invalid, 0, 0, NULL, NULL, NULL, 0 }
},
{
 { 4, 164, 80, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "<- 2", PrevScreen, NULL, Effect_Left },
 { 88, 164, 80, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "Exit", DisconnectFrontend, (void*)1, 1 },
 { 172, 164, 80, 16, libskin_BC_Normal, libskin_BS_Normal, 0, "1 ->", NextScreen, NULL, Effect_Right },

 { 4, 4, 120, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "MainMenu", SendCommand, CommandText[JUMP_MAINMENU], 0 },
 { 132, 4, 120, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "LiveTV", SendCommand, CommandText[JUMP_LIVETV], 0 },

 { 4, 44, 120, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "Music", SendCommand, CommandText[JUMP_PLAYMUSIC], 0 },
 { 132, 44, 120, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "Recordings", SendCommand, CommandText[JUMP_RECORDINGS], 0 },

 { 4, 84, 120, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "PlayDVD", SendCommand, CommandText[JUMP_PLAYDVD], 0 },
 { 132, 84, 120, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "Videos", SendCommand, CommandText[JUMP_VIDEOGALLERY], 0 },

 { 4, 124, 120, 32, libskin_BC_Normal, libskin_BS_Normal, 0, "Gallery", SendCommand, CommandText[JUMP_MYTHGALLERY], 0 },

 { 0, 0, 0, 0, libskin_BC_Invalid, 0, 0, NULL, NULL, NULL, 0 }
}
};

ButtonPanel_t RemotePanel[4] = {
{
	0,	// Accept Cursor Input
	0,	// Select 1st entry
	-1,	// TouchState
	0,	// Frames
	0,	// Bool Exit
	Effect_Down,
	TouchButtons[0]
},
{
	0,	// Accept Cursor Input
	0,	// Select 1st entry
	-1,	// TouchState
	0,	// Frames
	0,	// Bool Exit
	Effect_Down,
	TouchButtons[1]
},
{
	0,	// Accept Cursor Input
	0,	// Select 1st entry
	-1,	// TouchState
	0,	// Frames
	0,	// Bool Exit
	Effect_Down,
	TouchButtons[2]
},
{
	0,	// Accept Cursor Input
	0,	// Select 1st entry
	-1,	// TouchState
	0,	// Frames
	0,	// Bool Exit
	Effect_Down,
	TouchButtons[3]
}
};

int LocalPort=12400;
int ConnectFrontend(Host_t *Frontend) {
	int i=0/*,z=0*/;
	struct sockaddr_in addr = { 0 };

//	Button_t CancelButton = {
//		96,156,64,16,libskin_BC_Normal, libskin_BS_Normal, 0, "Cancel", NULL, NULL, 0
//	};

	tcp_sock = socket(AF_INET,SOCK_STREAM,0);
	if(tcp_sock == -1) {
		Do_Dialog("Failed to allocate socket.");
		return 1;
	}

	addr.sin_family = AF_INET;
	addr.sin_port = htons(LocalPort++);

	if(bind(tcp_sock, (struct sockaddr *) &addr, sizeof addr) == -1) {
		Do_Dialog("Failed to bind socket.");
		return 1;
	}

	WaitVbl();
	ClsBtm();

	BlitCenterText(&Skin, 128, "Connecting to MythTV...", Skin.Button.Highlight, BottomScreen);

	tcp_sain.sin_addr.s_addr=htonl(Frontend->IP);;
	tcp_sain.sin_port=htons(Frontend->Port);

//	i=1;
//	ioctl(tcp_sock,FIONBIO,&i); // set non-blocking

	if(connect(tcp_sock,(struct sockaddr *)&tcp_sain,sizeof(tcp_sain))) {
		Do_Dialog("Connection failed.");
		close(tcp_sock);
		return 1;
	}

	i=1;
	ioctl(tcp_sock,FIONBIO,&i); // set non-blocking

#if 0
	int frame=0;

	do {
		i = gettcpstate(tcp_sock);

		frame++;
		WaitVbl();
		ClsBtm();
		// Process Input
		if(keysDown()&KEY_B) {
			close(tcp_sock);
			return 1;
		}
		if(keysDown()&KEY_TOUCH) {
			// "cancel" - center of (16,21) to center of (31,23)
			if(touchXY.px>=(96) && touchXY.px<(96+64) && touchXY.py>=(156) && touchXY.py<(156+16)) {
				close(tcp_sock);
				return 1;
			}
		}

		// Draw info
		BlitCenterText(&Skin, 8, "Connecting to MythTV...", Skin.Button.Highlight, BottomScreen);

		DrawButton(&Skin, &CancelButton, BottomScreen);
		BlitCenterText(&Skin, 184,"[B]Cancel", Skin.Button.Highlight, BottomScreen);

		Blit_t BlockRect = { 0, 24*((frame>>2)&0x7), 24, 24, 24, (unsigned short *) WAITGFX_bin };
		Blit_t ScreenRect = { 116, 112, 24, 24, 256, BottomScreen };
		MyBlitter(&BlockRect, &ScreenRect);
		if(i!=z) {
			printf("i=%i\n", i);
			z=i;
		}

		if(i==SGIP_TCP_STATE_CLOSED || i==SGIP_TCP_STATE_LISTEN || i==SGIP_TCP_STATE_NODATA) { 
			Do_Dialog("Socket Error\n");
			close(tcp_sock);
			return 1;
		}
	} while(i!=SGIP_TCP_STATE_ESTABLISHED);
#endif
	
	return 0;
}

int SendFailed=0;

void SendCommand(void *arg) {
	char *cmd = (char*) arg;
	int sb=send(tcp_sock, cmd, strlen(cmd),0);
	if(sb<0) SendFailed=1;
}

int RemoteLoop=0;

// how=0 means we lost the connection
// how=1 means user quit.
void DisconnectFrontend(void *arg) {
	int how = (int)arg;
	int i;

	if(DEBUG) iprintf("Disconnect()\n");
	if(tcp_sock>=0) {
		if(how) {
			if(DEBUG) iprintf("SendCommand(\"exit\")\n");
			SendCommand(CommandText[CMD_EXIT]);
			for(i=0; i<15; i++)
				WaitVbl();

			if(DEBUG) iprintf("shutdown(tcp_sock, SHUT_RDWR)\n");
			shutdown(tcp_sock, SHUT_RDWR);

			for(i=0; i<15; i++)
				WaitVbl();
		}
		if(DEBUG) iprintf("close(tcp_sock)\n");
		close(tcp_sock);
	}
	tcp_sock=-1;

	if(DEBUG) iprintf("Wifi_DisconnectAP()\n");
	Wifi_DisconnectAP();

	if(DEBUG) iprintf("Wifi_DisableWifi()\n");
	Wifi_DisableWifi();

	RemoteLoop=0;
}

int ScreenNumber=0;
int NewScreenNumber=0;
int ScreenTransition=0;

// FIXME: Add 3D cube rotation animation here.
void FlipScreen() {
	RemotePanel[ScreenNumber].Selected=-1;
	RemotePanel[NewScreenNumber].Selected=-1;
	Transition(ScreenTransition | Effect_Out, &RemotePanel[ScreenNumber]);
	Transition(ScreenTransition | Effect_In, &RemotePanel[NewScreenNumber]);

	ScreenNumber=NewScreenNumber;

	WaitVbl();
	DrawBackdrop(&Skin, BottomScreen);
	RenderMenu(&RemotePanel[ScreenNumber], BottomScreen);
}

void PrevScreen(void *arg) {
	if(NewScreenNumber==0) NewScreenNumber=REMOTEPAGES-1;
	else NewScreenNumber--;
}

void NextScreen(void *arg) {
	NewScreenNumber++;
	if(NewScreenNumber>=REMOTEPAGES) NewScreenNumber=0;
}



void Do_Remote(void *param) {
	Host_t *Frontend = (Host_t *)param;
	ButtonPanel_t *pPanel = &RemotePanel[0];
	Button_t *pButtonList = pPanel->ButtonList;
	Button_t *pSelected = NULL;
	int i;

	pPanel->Exit=0;
	if(pPanel->CursorInput)
		pPanel->Selected=0;
	else
		pPanel->Selected=-1;
	pPanel->TouchState=-1;

	NewScreenNumber=ScreenNumber=0;
	SendFailed=0;
	
	if(Do_ConfigureWifi() < 1) return;

	if(ConnectFrontend(Frontend)) {
		return;
	}

	iprintf("Connected\n");

//	Transition(pPanel->Transition | Effect_In, pPanel);


	RemotePanel[ScreenNumber].Selected=-1;

	DrawBackdrop(&Skin, BackBuffer);
	RenderMenu(&RemotePanel[ScreenNumber], BackBuffer);

	WaitVbl();
	dmaCopy(BackBuffer, BottomScreen, 256*192*2);

	RemoteLoop=1;

	while(!pPanel->Exit && RemoteLoop) {
		DrawBackdrop(&Skin, BackBuffer);
		RenderMenu(&RemotePanel[ScreenNumber], BackBuffer);

		WaitVbl();

		dmaCopy(BackBuffer, BottomScreen, 256*192*2);

		while(!pPanel->Exit && RemoteLoop) {
			char discard[256];
			int rb=recv(tcp_sock, discard, 255, 0);
			if(rb>0) {
				discard[rb]=0;
				if(DEBUG) iprintf("%s", discard);
			}

//			i = gettcpstate(tcp_sock);

			if(SendFailed/* || i>=SGIP_TCP_STATE_FIN_WAIT_1*/) {
				if(!screen_on) {
					// Turn screens on.
					powerON(POWER_LCD);
					screen_on=1;
				}
				Do_Dialog("Socket disconnected.");
				DisconnectFrontend((void*)1);
				return;
			}		

			if(NewScreenNumber!=ScreenNumber) {
				FlipScreen();
				pPanel=&RemotePanel[ScreenNumber];
			}

			WaitVbl();
			
			unsigned long keys=keysDown();
			// Process input
			if(keys&KEY_UP) {
				SendCommand(CommandText[CMD_UP]);
			}
			if(keys&KEY_DOWN) {
				SendCommand(CommandText[CMD_DOWN]);
			}
			if(keys&KEY_LEFT) {
				SendCommand(CommandText[CMD_LEFT]);
			}
			if(keys&KEY_RIGHT) {
				SendCommand(CommandText[CMD_RIGHT]);
			}
			if(keys&KEY_A) {
				SendCommand(CommandText[CMD_SELECT]);
			}
			if(keys&KEY_B) {
				SendCommand(CommandText[CMD_BACK]);
			}
			if(keys&KEY_X) {
				SendCommand(CommandText[CMD_VOLUP]);
			}
			if(keys&KEY_Y) {
				SendCommand(CommandText[CMD_VOLDOWN]);
			}
			if(keys&KEY_SELECT) {
				if(!screen_on) {
					// Turn screens on.
					powerON(POWER_LCD);
					screen_on=1;
				}
				DisconnectFrontend((void*)1);
				RemoteLoop=0;
			}
			if(InputMenu(pPanel)) break;
		}
		while(keysHeld()&~KEY_LID)
			WaitVbl();

		pPanel = &RemotePanel[ScreenNumber];
		pButtonList = pPanel->ButtonList;

		pPanel->Frames=0;
		if(pPanel->Selected>=0) {
			pSelected = &pButtonList[pPanel->Selected];
			ScreenTransition = pSelected->Transition;
			if(pSelected->Function!=NULL)
				pSelected->Function(pSelected->Argument);
		}

		while(keysHeld()&~KEY_LID)
			WaitVbl();
	}
	Transition(AntiEffect(pPanel->Transition) | Effect_Out, pPanel);
}
