// MythRemote - copyright David Kuder (GrizzlyAdams) 2007
// Based on DSWifi Test Project - copyright Stephen Stair 2005
// Based on template project in ndslib by wntrmute
/****************************************************************************** 
DSWifi Lib and test materials are licenced under the MIT open source licence:
Copyright (c) 2005 Stephen Stair

MythRemote is also licensed under the MIT open source license:
Copyright (c) 2007 David Kuder

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
******************************************************************************/

#include <nds.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <ctype.h>

#include "libini.h"

libini_Section_t * libini_FindSection(libini_File_t * inifile, const char *sectionname) {
	libini_Section_t *cs;
	if(!inifile) return NULL;
	cs = inifile->Sections;
	while(cs) {
		if(!stricmp(cs->Name, sectionname)) return cs;
		cs=cs->Next;
	}
//	iprintf("libini_: Unable to find section '%s'\n", sectionname);
	return NULL;
}

libini_Key_t * libini_FindKey(libini_File_t * inifile, const char *sectionname, const char *keyname) {
	libini_Section_t *cs;
	libini_Key_t *ck;

	if(!inifile) return NULL;
	cs = libini_FindSection(inifile,sectionname);
	if(!cs) return NULL;

	ck = cs->Keys;
	while(ck) {
		if(!stricmp(ck->Name, keyname)) return ck;
		ck=ck->Next;
	}
//	iprintf("libini_: Unable to find key '%s'\n", keyname);
	return NULL;
}

libini_Section_t * libini_AddSection(libini_File_t * inifile, const char *sectionname) {
	libini_Section_t *ns, *cs;

	if(!inifile) return NULL;

	ns = (libini_Section_t*)malloc(sizeof(libini_Section_t));
	ns->Name = strdup(sectionname);
	ns->Keys = NULL;
	ns->Next = NULL;

	if(inifile->Sections) {
		cs = inifile->Sections;
		while(cs->Next) cs=cs->Next;
		cs->Next = ns;
	} else {
		inifile->Sections = ns;
	}
//	iprintf("libini_: Added section '%s'\n", sectionname);

	return ns;
}

libini_Key_t * libini_AddKey(libini_File_t * inifile, const char *sectionname, const char *keyname, char *value) {
	libini_Section_t *cs;
	libini_Key_t *nk, *ck;

	if(!inifile) return NULL;

	cs = libini_FindSection(inifile,sectionname);
	if(!cs) cs = libini_AddSection(inifile,sectionname);
	if(!cs) return NULL;

	nk=(libini_Key_t*)malloc(sizeof(libini_Key_t));
	if(!nk) return NULL;

	nk->Name = strdup(keyname);
	nk->Value = strdup(value);
	nk->Next = NULL;

	if(cs->Keys) {
		ck = cs->Keys;
		while(ck->Next) {
			ck=ck->Next;
		}
		ck->Next = nk;
	} else {
		cs->Keys = nk;
	}

	return nk;
}

void libini_FreeSection(libini_Section_t *section) {
	libini_Key_t *ck, *cn;
	if(!section) return;
	ck = section->Keys;
	while(ck) {
		cn=ck->Next;
		free(ck->Name);
		free(ck->Value);
		free(ck);
		ck=cn;
	}
	free(section);
}

void libini_Free(libini_File_t * inifile) {
	libini_Section_t *cs, *cn;
	if(!inifile) return;
	cs = inifile->Sections;
	while(cs) {
		cn=cs->Next;
		libini_FreeSection(cs);
		cs=cn;
	}
	free(inifile);
}

libini_File_t * libini_Create() {
	libini_File_t *inifile =(libini_File_t *) malloc(sizeof(libini_File_t));
	if(inifile) memset(inifile, 0, sizeof(libini_File_t));
	return inifile;
}

static int strncatchr(char *str, int max, char data) {
	while(*str && max) {
		str++;
		max--;
	}

	if(max<=0) return -1;

	*str++=data;
	*str=0;

	return 0;
}

static int ftotal(FILE *f) {
	fpos_t pos;
	int total;

	if(fgetpos(f, &pos)!=0) return -1;

	if(fseek(f, 0, SEEK_END)!=0) {
		if(fsetpos(f, &pos)!=0) return -2;
		return -1;
	}
	total=ftell(f);

	if(fsetpos(f, &pos)!=0) return -2;

	return total;
}

libini_File_t * libini_Load(const char *filename) {
	FILE *fd;
	char lastchar=0;
	int epos,i;
	libini_File_t *inifile = NULL;
	char line[1024], lastsection[128];

	fd = fopen(filename, "r");
	if(fd==NULL) return NULL;

	memset(line, 0, 1024);
	memset(lastsection, 0, 64);
	strcpy(lastsection, "DEFAULT");

	inifile = libini_Create();
	if(!inifile) goto errorout;

	while(ftell(fd) < ftotal(fd)) {
//		iprintf("Read: ");
		lastchar = 0;
		while( (lastchar != '\r') && (lastchar != '\n') ) {
			fread(&lastchar, 1, 1, fd);
			if((strlen(line)<1022) && (lastchar != 0) && (lastchar != '\r') && (lastchar != '\n') ) {
//				iprintf("%c", lastchar);
				strncatchr(line, 1022, lastchar);
			}
		}
//		iprintf("\n");
		switch(line[0]) {
			case ';': // Comment
//				iprintf("Comment: %s\n", line);
				libini_AddKey(inifile, lastsection, ";", line);
				break;
			case '[':
//				iprintf("Section: %s\n", line);
				for( i=0; line[i]; i++) {
					if(line[i]==']') {
						line[i]=0;
						break;
					}
				}
				libini_AddSection(inifile, line+1);
				strcpy(lastsection, line+1);
				break;
			default:
//				iprintf("Data: %s\n", line);
				for(epos = 0; (line[epos] != '=') && (line[epos] != 0); epos++);
				if(line[epos] != 0) {
					line[epos]=0;
					epos++;
//					iprintf(" AddKey: %s=%s\n", line, line+epos);
					libini_AddKey(inifile, lastsection, line, line+epos);
				}
		}
		memset(line, 0, 1024);
	}

	fclose(fd);

	return inifile;

errorout:
	if(inifile) free(inifile);
	return NULL;
}

int libini_Save(libini_File_t * inifile, const char *filename) {
	FILE *fd;
	libini_Section_t *cs;
	libini_Key_t *ck;
	if((!inifile)||(!filename)) return 0;

	fd = fopen(filename, "w");
	if(fd!=NULL) {
		for(cs=inifile->Sections; cs; cs=cs->Next) {
			if(stricmp(cs->Name, "DEFAULT")) {
				fputs("\r\n[", fd);
				fputs(cs->Name, fd);
				fputs("]\r\n", fd);
			}
			for(ck=cs->Keys; (ck); ck=ck->Next) {
				if(ck->Name[0]==';') {
					fputs(ck->Value, fd);
					fputs("\r\n", fd);
				} else {
					fputs(ck->Name, fd);
					fputs("=", fd);
					fputs(ck->Value, fd);
					fputs("\r\n", fd);
				}
			}
			fputs("\r\n", fd);
		}
		fclose(fd);
	}
	
	return 0;
}

char * libini_ReadString(libini_File_t * inifile, const char *sectionname, const char *keyname, char *def) {
	libini_Key_t *key = libini_FindKey(inifile, sectionname, keyname);

	if(key) {
		return key->Value;
	}

	return def;
}

int libini_ReadInt(libini_File_t * inifile, const char *sectionname, const char *keyname, int def) {
	char *text = libini_ReadString(inifile, sectionname, keyname, NULL);
	int rv;

	if(text) {
		rv = atoi(text);
		return rv;
	}
	
	return def;
}

static unsigned long stoul(char *text) {
	char *inb = text;
	unsigned long rv = 0;
	unsigned int base = 10;
	int i, inl;

	if(text[0] == '0') {
		if(text[1] == 'x') {
			inb = text+2;
			base = 16;
		} else base = 8;
	} else if(text[0] == 'b')
		base = 2;
	else base = 10;

	inl = strlen(inb);

//	printf("parsing: %s\n", text);
	for(i=0; i<inl; i++) {
//		printf("%c=%d=", inb[i], inb[i]);
		rv*=base;
		switch(inb[i]) {
			case 'a': case 'b': case 'c': case 'd': case 'e': case 'f':
				rv |= (inb[i] - 'a') + 0xa;
				break;
			case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
				rv |= (inb[i] - 'A') + 0xa;
				break;
			case '0': case '1': case '2': case '3': case '4':
			case '5': case '6': case '7': case '8': case '9': 
				rv |= (inb[i] - '0');
				break;
		}
//		printf("0x%lx  ",rv % base);
	}
//	printf("0x%lx\n",rv);

	return rv;
}

unsigned long libini_ReadULong(libini_File_t * inifile, const char *sectionname, const char *keyname, unsigned long def) {
	char *text = libini_ReadString(inifile, sectionname, keyname, NULL);
	unsigned long rv;

	if(text) {
		rv = stoul(text);
		return rv;
	}
	
	return def;
}

libini_Key_t * libini_WriteString(libini_File_t * inifile, const char *sectionname, const char *keyname, char *value) {
	libini_Key_t *key = libini_FindKey(inifile, sectionname, keyname);

	if(!key) return libini_AddKey(inifile, sectionname, keyname, value);

	if(key->Value) free(key->Value);
	key->Value = strdup(value);

	return key;
}

static char *ltoa(long value, char *buf, int maxlen) {
	long v = value;
	int i;

	if(maxlen<2) return NULL;

	buf[maxlen-1] = 0;
	for(i=maxlen-2; i>=0; i--) {
		buf[i]=(char)('0'+(v%10));
		v/=10;
		if(v==0) break;
	}

	if(i>0) strcpy(buf, buf+i);
	else return NULL; // buffer not long enough.

	return buf;
}

libini_Key_t * libini_WriteInt(libini_File_t * inifile, const char *sectionname, const char *keyname, int value) {
	libini_Key_t *rv;
	char *text = malloc(64);

	if(text) {
		ltoa(value, text, 63);
		rv = libini_WriteString(inifile, sectionname, keyname, text);
		free(text);
		return rv;
	}
	
	return NULL;
}
