// MythRemote - copyright David Kuder (GrizzlyAdams) 2007
// Based on DSWifi Test Project - copyright Stephen Stair 2005
// Based on template project in ndslib by wntrmute
/****************************************************************************** 
DSWifi Lib and test materials are licenced under the MIT open source licence:
Copyright (c) 2005 Stephen Stair

MythRemote is also licensed under the MIT open source license:
Copyright (c) 2007 David Kuder

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
******************************************************************************/

#include <nds.h>
#include <string.h>

#include "libskin.h"


void MyBlitter(Blit_t *SRC, Blit_t *DST) {
	int x, y;
	unsigned short *pSRC, *pDST;

#ifdef SAFE_N_SLOW
	if(SRC==NULL) return;
	if(SRC->Image==NULL) return;
	if(DST==NULL) return;
	if(DST->Image==NULL) return;
#endif

	pSRC = SRC->Image+SRC->X+SRC->Y*SRC->LW;
	pDST = DST->Image+DST->X+DST->Y*DST->LW;

	for (y=0; y<SRC->H; y++) {
		for (x=0; x<SRC->W; x++) {
			unsigned short COL=pSRC[x+y*SRC->LW];
			if(COL&0x8000)
				pDST[x+y*DST->LW] = COL | BIT(15);
		}
	}
}

void BlitText(Skin_t *Skin, int oX, int oY, char *Text, unsigned short Color, unsigned short *Screen) {
	int x,y,i;
	unsigned short *pOut;

#ifdef SAFE_N_SLOW
	if(Skin==NULL) return;
	if(Text==NULL) return;
	if(Screen==NULL) return;
#endif

	pOut=&(Screen[oY*256+oX]);

	for(i=0; Text[i]; i++) {
		for(y=0; y<8; y++) {
			for(x=0; x<8; x++) {
				int Character = Text[i];
				Character*=64;
				if(Skin->Font[Character+y*8+x]) pOut[i*8+x+y*256]=Color;
//				else pOut[i*8+x+y*256]=0x0000; // DEBUG
			}
		}
	}
}

void BlitCenterText(Skin_t *Skin, int oY, char *Text, unsigned short Color, unsigned short *Screen) {
	int x;
#ifdef SAFE_N_SLOW
	if(Skin==NULL) return;
	if(Text==NULL) return;
	if(Screen==NULL) return;
#endif
	x=(256-strlen(Text)*8)/2;
	BlitText(Skin, x, oY, Text, Color, Screen);
}


static void DrawHLine(int X, int Y, int W, unsigned short Color, unsigned short *Screen) {
	int x;
	for(x=0; x<W; x++) Screen[x+X+Y*256]=Color;
}

static void DrawVLine(int X, int Y, int H, unsigned short Color, unsigned short *Screen) {
	int y;
	for(y=0; y<H; y++) Screen[X+(Y+y)*256]=Color;
}

void DrawButton(Skin_t *Skin, Button_t *Button, unsigned short *Screen) {
	int ox,oy;
	Blit_t SRC,DST;

#ifdef SAFE_N_SLOW
	if(Skin==NULL) return;
	if(Button==NULL) return;
	if(Screen==NULL) return;
#endif

	DST.Image=Screen;
	DST.X=Button->X;
	DST.Y=Button->Y;
	DST.LW=256;

	SRC.Image=Skin->Button.Image[Button->State];
	switch(Skin->Button.AlignX) {
		case libskin_IAX_Direct:	SRC.X=DST.X;			break;
		case libskin_IAX_Left:		SRC.X=0;			break;
		case libskin_IAX_Right:		SRC.X=256-Button->W;		break;
		case libskin_IAX_Center:	SRC.X=(256-Button->W)/2;	break;
	}
	switch(Skin->Button.AlignY) {
		case libskin_IAY_Direct:	SRC.Y=DST.Y;			break;
		case libskin_IAY_Top:		SRC.Y=0;			break;
		case libskin_IAY_Bottom:	SRC.Y=32-Button->H;		break;
		case libskin_IAY_Center:	SRC.Y=(32-Button->H)/2;	break;
	}
	SRC.W=Button->W;
	SRC.H=Button->H;
	SRC.LW=256;

	MyBlitter(&SRC, &DST);

	if(Button->State & libskin_BS_Pressed) {
		DrawHLine(Button->X, Button->Y, Button->W, Skin->Button.Shadow, Screen); 
		DrawVLine(Button->X, Button->Y, Button->H, Skin->Button.Shadow, Screen); 
		DrawHLine(Button->X, Button->Y+Button->H-1, Button->W, Skin->Button.Highlight, Screen); 
		DrawVLine(Button->X+Button->W-1, Button->Y, Button->H, Skin->Button.Highlight, Screen); 
	} else {
		DrawHLine(Button->X, Button->Y, Button->W, Skin->Button.Highlight, Screen); 
		DrawVLine(Button->X, Button->Y, Button->H, Skin->Button.Highlight, Screen); 
		DrawHLine(Button->X, Button->Y+Button->H-1, Button->W, Skin->Button.Shadow, Screen); 
		DrawVLine(Button->X+Button->W-1, Button->Y, Button->H, Skin->Button.Shadow, Screen); 
	}
	if(Button->State & libskin_BS_Highlight) {
		DrawHLine(Button->X+1, Button->Y+1, Button->W-2, Skin->Button.Highlight, Screen); 
		DrawVLine(Button->X+1, Button->Y+1, Button->H-2, Skin->Button.Highlight, Screen); 
		DrawHLine(Button->X+1, Button->Y+Button->H-2, Button->W-2, Skin->Button.Shadow, Screen); 
		DrawVLine(Button->X+Button->W-2, Button->Y+1, Button->H-2, Skin->Button.Shadow, Screen); 
	}
	oy=(Button->H-8)/2;

	if(Button->Class==libskin_BC_CheckBox) {
		SRC.Image=Skin->Check[Button->Checked];
		SRC.X=0; SRC.W=12;
		SRC.Y=0; SRC.H=12;
		SRC.LW=12;
		ox=20+((Button->W-20)-(strlen(Button->Label)*8))/2;
		DST.X=Button->X+ox-20; DST.W=16;
		DST.Y=Button->Y+2; DST.H=16;
		DST.LW=256;
		MyBlitter(&SRC, &DST);
	} else if(Button->Class & libskin_BC_RadioGroup) {
		SRC.Image=Skin->Radio[Button->Checked];
		SRC.X=0; SRC.W=12;
		SRC.Y=0; SRC.H=12;
		SRC.LW=12;
		ox=20+((Button->W-20)-(strlen(Button->Label)*8))/2;
		DST.X=Button->X+ox-20; DST.W=16;
		DST.Y=Button->Y+2; DST.H=16;
		DST.LW=256;
		MyBlitter(&SRC, &DST);
	} else {
		ox=(Button->W-(strlen(Button->Label)*8))/2;
	}

	BlitText(Skin, Button->X+ox, Button->Y+oy, Button->Label, Skin->TextColor, Screen);
}

void DrawButtonPanel(Skin_t *Skin, ButtonPanel_t *Panel, unsigned short *Screen) {
	int i;

#ifdef SAFE_N_SLOW
	if(Skin==NULL) return;
	if(Panel==NULL) return;
	if(Screen==NULL) return;
#endif

	for (i=0; Panel->ButtonList[i].Class!=libskin_BC_Invalid; i++) {
		Button_t *pButton=&(Panel->ButtonList[i]);
		if(Panel->Selected==i)	pButton->State|=libskin_BS_Highlight;
		else			pButton->State&=~libskin_BS_Highlight;
		DrawButton(Skin, pButton, Screen);
	}
}

void DrawBackdrop(Skin_t *Skin, unsigned short *Screen) {
#ifdef SAFE_N_SLOW
	if(Skin==NULL) return;
	if(Screen==NULL) return;
#endif
	int x,y;
	for(y=0; y<192; y++)
	for(x=0; x<256; x++)
	Screen[y*256+x] = Skin->Backdrop[y*256+x];
}


